@@ -32,7 +32,6 @@
 
 #include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
-#include <gconf/gconf-client.h>
 
 #include <telepathy-glib/util.h>
 
@@ -49,7 +48,7 @@
 #include <libempathy/empathy-debug.h>
 
 /* Number of seconds between timestamps when using normal mode, 5 minutes. */
-#define TIMESTAMP_INTERVAL 300
+#define TIMESTAMP_INTERVAL (5 * G_TIME_SPAN_MINUTE)
 
 #define MAX_LINES 800
 #define MAX_SCROLL_TIME 0.4 /* seconds */
@@ -66,10 +65,11 @@ typedef struct {
 	gboolean              find_wrapped;
 	gboolean              find_last_direction;
 	EmpathyContact       *last_contact;
-	time_t                last_timestamp;
+	gint64                last_timestamp;
 	gboolean              allow_scrolling;
 	guint                 notify_system_fonts_id;
-	GConfClient          *gconf_client;
+	GSettings            *gsettings_desktop;
+	GSettings            *gsettings_chat;
 	EmpathySmileyManager *smiley_manager;
 	gboolean              only_if_date;
 } EmpathyChatTextViewPriv;
@@ -209,9 +209,8 @@ chat_text_view_system_font_update (EmpathyChatTextView *view)
 	PangoFontDescription *font_description = NULL;
 	gchar                *font_name;
 
-	font_name = gconf_client_get_string (priv->gconf_client,
-			"/desktop/gnome/interface/document_font_name",
-			NULL);
+	font_name = g_settings_get_string (priv->gsettings_desktop,
+			EMPATHY_PREFS_DESKTOP_INTERFACE_DOCUMENT_FONT_NAME);
 
 	if (font_name != NULL) {
 		font_description = pango_font_description_from_string (font_name);
@@ -220,7 +219,7 @@ chat_text_view_system_font_update (EmpathyChatTextView *view)
 		font_description = NULL;
 	}
 
-	gtk_widget_modify_font (GTK_WIDGET (view), font_description);
+	gtk_widget_override_font (GTK_WIDGET (view), font_description);
 
 	if (font_description) {
 		pango_font_description_free (font_description);
@@ -228,14 +227,11 @@ chat_text_view_system_font_update (EmpathyChatTextView *view)
 }
 
 static void
-chat_text_view_notify_system_font_cb (GConfClient *conf,
-				      guint id,
-				      GConfEntry *entry,
-				      gpointer user_data)
+chat_text_view_notify_system_font_cb (GSettings *gsettings,
+				      const gchar *key,
+				      EmpathyChatTextView *self)
 {
-	EmpathyChatTextView *view = user_data;
-
-	chat_text_view_system_font_update (view);
+	chat_text_view_system_font_update (self);
 }
 
 static void
@@ -274,11 +270,11 @@ chat_text_view_populate_popup (EmpathyChatTextView *view,
 	/* Clear menu item */
 	if (gtk_text_buffer_get_char_count (priv->buffer) > 0) {
 		item = gtk_separator_menu_item_new ();
-		gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item);
+		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 		gtk_widget_show (item);
 
 		item = gtk_image_menu_item_new_from_stock (GTK_STOCK_CLEAR, NULL);
-		gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item);
+		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 		gtk_widget_show (item);
 
 		g_signal_connect_swapped (item, "activate",
@@ -344,7 +340,7 @@ chat_text_view_is_scrolled_down (EmpathyChatTextView *view)
 	gdouble upper;
 	gdouble page_size;
 
-	vadj = gtk_text_view_get_vadjustment (GTK_TEXT_VIEW (view));
+	vadj = gtk_scrollable_get_vadjustment (GTK_SCROLLABLE (view));
 	value = gtk_adjustment_get_value (vadj);
 	upper = gtk_adjustment_get_upper (vadj);
 	page_size = gtk_adjustment_get_page_size (vadj);
@@ -402,7 +398,7 @@ chat_text_view_maybe_trim_buffer (EmpathyChatTextView *view)
 
 static void
 chat_text_view_append_timestamp (EmpathyChatTextView *view,
-				 time_t               timestamp,
+				 gint64               timestamp,
 				 gboolean             show_date)
 {
 	EmpathyChatTextViewPriv *priv = GET_PRIV (view);
@@ -414,17 +410,12 @@ chat_text_view_append_timestamp (EmpathyChatTextView *view,
 
 	/* Append date if needed */
 	if (show_date) {
-		GDate *date;
-		gchar  buf[256];
-
-		date = g_date_new ();
-		g_date_set_time_t (date, timestamp);
 		/* Translators: timestamp displayed between conversations in
 		 * chat windows (strftime format string) */
-		g_date_strftime (buf, 256, _("%A %B %d %Y"), date);
-		g_string_append (str, buf);
+		tmp = empathy_time_to_string_utc (timestamp, _("%A %B %d %Y"));
+		g_string_append (str, tmp);
 		g_string_append (str, ", ");
-		g_date_free (date);
+		g_free (tmp);
 	}
 
 	/* Append time */
@@ -448,32 +439,32 @@ chat_text_view_append_timestamp (EmpathyChatTextView *view,
 
 static void
 chat_text_maybe_append_date_and_time (EmpathyChatTextView *view,
-				      time_t               timestamp)
+				      gint64               timestamp)
 {
 	EmpathyChatTextViewPriv *priv = GET_PRIV (view);
-	GDate                   *date, *last_date;
+	GDateTime               *date, *last_date;
 	gboolean                 append_date = FALSE;
 	gboolean                 append_time = FALSE;
+	GTimeSpan                delta;
 
 	/* Get the date from last message */
-	last_date = g_date_new ();
-	g_date_set_time_t (last_date, priv->last_timestamp);
+	last_date = g_date_time_new_from_unix_utc (priv->last_timestamp);
 
 	/* Get the date of the message we are appending */
-	date = g_date_new ();
-	g_date_set_time_t (date, timestamp);
+	date = g_date_time_new_from_unix_utc (timestamp);
 
+	delta = g_date_time_difference (last_date, date);
 	/* If last message was from another day we append date and time */
-	if (g_date_compare (date, last_date) > 0) {
+	if (delta >= G_TIME_SPAN_DAY) {
 		append_date = TRUE;
 		append_time = TRUE;
 	}
 
-	g_date_free (last_date);
-	g_date_free (date);
+	g_date_time_unref (last_date);
+	g_date_time_unref (date);
 
 	/* If last message is 'old' append the time */
-	if (timestamp - priv->last_timestamp >= TIMESTAMP_INTERVAL) {
+	if (delta >= TIMESTAMP_INTERVAL) {
 		append_time = TRUE;
 	}
 
@@ -495,7 +486,7 @@ chat_text_view_size_allocate (GtkWidget     *widget,
 	if (down) {
 		GtkAdjustment *adj;
 
-		adj = gtk_text_view_get_vadjustment (GTK_TEXT_VIEW (widget));
+		adj = gtk_scrollable_get_vadjustment (GTK_SCROLLABLE (widget));
 		gtk_adjustment_set_value (adj,
 					  gtk_adjustment_get_upper (adj) -
 					  gtk_adjustment_get_page_size (adj));
@@ -565,9 +556,8 @@ chat_text_view_finalize (GObject *object)
 
 	DEBUG ("%p", object);
 
-	gconf_client_notify_remove (priv->gconf_client,
-				    priv->notify_system_fonts_id);
-	g_object_unref (priv->gconf_client);
+	g_object_unref (priv->gsettings_chat);
+	g_object_unref (priv->gsettings_desktop);
 
 	if (priv->last_contact) {
 		g_object_unref (priv->last_contact);
@@ -642,16 +632,14 @@ empathy_chat_text_view_init (EmpathyChatTextView *view)
 		      "cursor-visible", FALSE,
 		      NULL);
 
-	priv->gconf_client = gconf_client_get_default ();
-	gconf_client_add_dir (priv->gconf_client,
-			      "/desktop/gnome/interface",
-			      GCONF_CLIENT_PRELOAD_ONELEVEL,
-			      NULL);
-	priv->notify_system_fonts_id =
-		gconf_client_notify_add (priv->gconf_client,
-					 "/desktop/gnome/interface/document_font_name",
-					 chat_text_view_notify_system_font_cb,
-					 view, NULL, NULL);
+	priv->gsettings_chat = g_settings_new (EMPATHY_PREFS_CHAT_SCHEMA);
+
+	priv->gsettings_desktop = g_settings_new (
+			  EMPATHY_PREFS_DESKTOP_INTERFACE_SCHEMA);
+	g_signal_connect (priv->gsettings_desktop,
+			  "changed::" EMPATHY_PREFS_DESKTOP_INTERFACE_DOCUMENT_FONT_NAME,
+			  G_CALLBACK (chat_text_view_notify_system_font_cb),
+			  view);
 	chat_text_view_system_font_update (view);
 	chat_text_view_create_tags (view);
 
@@ -683,7 +671,7 @@ chat_text_view_scroll_cb (EmpathyChatTextView *view)
 
 	priv = GET_PRIV (view);
 
-	adj = gtk_text_view_get_vadjustment (GTK_TEXT_VIEW (view));
+	adj = gtk_scrollable_get_vadjustment (GTK_SCROLLABLE (view));
 	max_val = gtk_adjustment_get_upper (adj) - gtk_adjustment_get_page_size (adj);
 
 	g_return_val_if_fail (priv->scroll_time != NULL, FALSE);
@@ -732,7 +720,7 @@ chat_text_view_append_message (EmpathyChatView *view,
 	EmpathyChatTextView     *text_view = EMPATHY_CHAT_TEXT_VIEW (view);
 	EmpathyChatTextViewPriv *priv = GET_PRIV (text_view);
 	gboolean                 bottom;
-	time_t                   timestamp;
+	gint64                   timestamp;
 
 	g_return_if_fail (EMPATHY_IS_CHAT_TEXT_VIEW (view));
 	g_return_if_fail (EMPATHY_IS_MESSAGE (msg));
@@ -1327,7 +1315,7 @@ empathy_chat_text_view_get_last_contact (EmpathyChatTextView *view)
 	return priv->last_contact;
 }
 
-time_t
+gint64
 empathy_chat_text_view_get_last_timestamp (EmpathyChatTextView *view)
 {
 	EmpathyChatTextViewPriv *priv = GET_PRIV (view);
@@ -1418,11 +1406,9 @@ empathy_chat_text_view_append_body (EmpathyChatTextView *view,
 	GtkTextIter              start_iter;
 	GtkTextIter              iter;
 	GtkTextMark             *mark;
-	GSettings		*gsettings_chat;
 
 	/* Check if we have to parse smileys */
-	gsettings_chat = g_settings_new (EMPATHY_PREFS_CHAT_SCHEMA);
-	use_smileys = g_settings_get_boolean (gsettings_chat,
+	use_smileys = g_settings_get_boolean (priv->gsettings_chat,
 			EMPATHY_PREFS_CHAT_SHOW_SMILEYS);
 
 	if (use_smileys)
@@ -1449,8 +1435,6 @@ empathy_chat_text_view_append_body (EmpathyChatTextView *view,
 					   &iter);
 
 	gtk_text_buffer_delete_mark (priv->buffer, mark);
-
-	g_object_unref (gsettings_chat);
 }
 
 void
