@@ -34,12 +34,14 @@
 #include <telepathy-glib/util.h>
 
 #include <libempathy/empathy-utils.h>
+#include <libempathy/empathy-tp-chat.h>
 #include <libempathy/empathy-enum-types.h>
 #include <libempathy/empathy-contact-manager.h>
 
 #include "empathy-contact-list-store.h"
 #include "empathy-ui-utils.h"
 #include "empathy-gtk-enum-types.h"
+#include "empathy-images.h"
 
 #define DEBUG_FLAG EMPATHY_DEBUG_CONTACT
 #include <libempathy/empathy-debug.h>
@@ -181,6 +183,30 @@ enum {
 
 G_DEFINE_TYPE (EmpathyContactListStore, empathy_contact_list_store, GTK_TYPE_TREE_STORE);
 
+static void
+contact_list_store_chat_state_changed_cb (TpChannel *self,
+				     guint contact_handle,
+				     guint state,
+				     gpointer store)
+{
+	EmpathyContactListStorePriv *priv = GET_PRIV (store);
+	GList *contacts, *l;
+
+	contacts = empathy_contact_list_get_members (priv->list);
+
+	for (l = contacts; l != NULL; l = l->next) {
+		EmpathyContact *contact = EMPATHY_CONTACT (l->data);
+
+		if (empathy_contact_get_handle (contact) == contact_handle) {
+			contact_list_store_contact_update (store, contact);
+			break;
+		}
+	}
+
+	g_list_foreach (contacts, (GFunc) g_object_unref, NULL);
+	g_list_free (contacts);
+}
+
 static gboolean
 contact_list_store_iface_setup (gpointer user_data)
 {
@@ -206,6 +232,20 @@ contact_list_store_iface_setup (gpointer user_data)
 			  G_CALLBACK (contact_list_store_groups_changed_cb),
 			  store);
 
+	if (EMPATHY_IS_TP_CHAT (priv->list)) {
+		TpChannel *channel;
+
+		channel = empathy_tp_chat_get_channel (EMPATHY_TP_CHAT (priv->list));
+		if (!tp_proxy_is_prepared (channel, TP_CHANNEL_FEATURE_CHAT_STATES)) {
+			DEBUG ("Chat state feature not prepared");
+		} else {
+			g_signal_connect (channel,
+					  "chat-state-changed",
+					  G_CALLBACK (contact_list_store_chat_state_changed_cb),
+					  store);
+		}
+	}
+
 	/* Add contacts already created. */
 	contacts = empathy_contact_list_get_members (priv->list);
 	for (l = contacts; l; l = l->next) {
@@ -411,10 +451,6 @@ contact_list_store_set_property (GObject      *object,
 				 const GValue *value,
 				 GParamSpec   *pspec)
 {
-	EmpathyContactListStorePriv *priv;
-
-	priv = GET_PRIV (object);
-
 	switch (param_id) {
 	case PROP_CONTACT_LIST:
 		contact_list_store_set_contact_list (EMPATHY_CONTACT_LIST_STORE (object),
@@ -604,7 +640,6 @@ empathy_contact_list_store_set_show_groups (EmpathyContactListStore *store,
 					    gboolean                 show_groups)
 {
 	EmpathyContactListStorePriv *priv;
-	GList                       *contacts, *l;
 
 	g_return_if_fail (EMPATHY_IS_CONTACT_LIST_STORE (store));
 
@@ -616,19 +651,28 @@ empathy_contact_list_store_set_show_groups (EmpathyContactListStore *store,
 
 	priv->show_groups = show_groups;
 
-	/* Remove all contacts and add them back, not optimized but that's the
-	 * easy way :) */
-	gtk_tree_store_clear (GTK_TREE_STORE (store));
-	contacts = empathy_contact_list_get_members (priv->list);
-	for (l = contacts; l; l = l->next) {
-		contact_list_store_members_changed_cb (priv->list, l->data,
-						       NULL, 0, NULL,
-						       TRUE,
-						       store);
+	if (priv->setup_idle_id == 0) {
+		/* Remove all contacts and add them back, not optimized but
+		 * that's the easy way :)
+		 *
+		 * This is only done if there's not a pending setup idle
+		 * callback, otherwise it will race and the contacts will get
+		 * added twice */
+		GList *contacts, *l;
 
-		g_object_unref (l->data);
+		gtk_tree_store_clear (GTK_TREE_STORE (store));
+		contacts = empathy_contact_list_get_members (priv->list);
+		for (l = contacts; l; l = l->next) {
+			contact_list_store_members_changed_cb (priv->list,
+							       l->data,
+							       NULL, 0, NULL,
+							       TRUE,
+							       store);
+
+			g_object_unref (l->data);
+		}
+		g_list_free (contacts);
 	}
-	g_list_free (contacts);
 
 	g_object_notify (G_OBJECT (store), "show-groups");
 }
@@ -704,6 +748,9 @@ empathy_contact_list_store_set_sort_criterium (EmpathyContactListStore     *stor
 						      EMPATHY_CONTACT_LIST_STORE_COL_NAME,
 						      GTK_SORT_ASCENDING);
 		break;
+
+	default:
+		g_assert_not_reached ();
 	}
 
 	g_object_notify (G_OBJECT (store), "sort-criterium");
@@ -782,48 +829,6 @@ empathy_contact_list_store_get_parent_group (GtkTreeModel *model,
 	return name;
 }
 
-gboolean
-empathy_contact_list_store_search_equal_func (GtkTreeModel *model,
-					      gint          column,
-					      const gchar  *key,
-					      GtkTreeIter  *iter,
-					      gpointer      search_data)
-{
-	gchar    *name, *name_folded;
-	gchar    *key_folded;
-	gboolean  ret;
-
-	g_return_val_if_fail (GTK_IS_TREE_MODEL (model), FALSE);
-
-	if (!key) {
-		return TRUE;
-	}
-
-	gtk_tree_model_get (model, iter,
-			    EMPATHY_CONTACT_LIST_STORE_COL_NAME, &name,
-			    -1);
-
-	if (!name) {
-		return TRUE;
-	}
-
-	name_folded = g_utf8_casefold (name, -1);
-	key_folded = g_utf8_casefold (key, -1);
-
-	if (name_folded && key_folded &&
-	    strstr (name_folded, key_folded)) {
-		ret = FALSE;
-	} else {
-		ret = TRUE;
-	}
-
-	g_free (name);
-	g_free (name_folded);
-	g_free (key_folded);
-
-	return ret;
-}
-
 static void
 contact_list_store_setup (EmpathyContactListStore *store)
 {
@@ -833,8 +838,9 @@ contact_list_store_setup (EmpathyContactListStore *store)
 		GDK_TYPE_PIXBUF,      /* Avatar pixbuf */
 		G_TYPE_BOOLEAN,       /* Avatar pixbuf visible */
 		G_TYPE_STRING,        /* Name */
+		G_TYPE_UINT,          /* Presence type */
 		G_TYPE_STRING,        /* Status string */
-		G_TYPE_BOOLEAN,       /* Show status */
+		G_TYPE_BOOLEAN,       /* Compact view */
 		EMPATHY_TYPE_CONTACT, /* Contact type */
 		G_TYPE_BOOLEAN,       /* Is group */
 		G_TYPE_BOOLEAN,       /* Is active */
@@ -920,10 +926,6 @@ contact_list_store_members_changed_cb (EmpathyContactList      *list_iface,
 				       gboolean                 is_member,
 				       EmpathyContactListStore *store)
 {
-	EmpathyContactListStorePriv *priv;
-
-	priv = GET_PRIV (store);
-
 	DEBUG ("Contact %s (%d) %s",
 		empathy_contact_get_id (contact),
 		empathy_contact_get_handle (contact),
@@ -942,10 +944,6 @@ contact_list_store_favourites_changed_cb (EmpathyContactList      *list_iface,
 					  gboolean                 is_favourite,
 					  EmpathyContactListStore *store)
 {
-	EmpathyContactListStorePriv *priv;
-
-	priv = GET_PRIV (store);
-
 	DEBUG ("Contact %s (%d) is %s a favourite",
 		empathy_contact_get_id (contact),
 		empathy_contact_get_handle (contact),
@@ -963,10 +961,6 @@ contact_list_store_member_renamed_cb (EmpathyContactList      *list_iface,
 				      gchar                   *message,
 				      EmpathyContactListStore *store)
 {
-	EmpathyContactListStorePriv *priv;
-
-	priv = GET_PRIV (store);
-
 	DEBUG ("Contact %s (%d) renamed to %s (%d)",
 		empathy_contact_get_id (old_contact),
 		empathy_contact_get_handle (old_contact),
@@ -1010,11 +1004,12 @@ contact_list_store_groups_changed_cb (EmpathyContactList      *list_iface,
 static void
 add_contact_to_store (GtkTreeStore *store,
 		      GtkTreeIter *iter,
+		      GtkTreeIter *parent,
 		      EmpathyContact *contact,
 		      EmpathyContactListFlags flags)
 {
-	gtk_tree_store_set (store, iter,
-			    EMPATHY_CONTACT_LIST_STORE_COL_NAME, empathy_contact_get_name (contact),
+	gtk_tree_store_insert_with_values (store, iter, parent, 0,
+			    EMPATHY_CONTACT_LIST_STORE_COL_NAME, empathy_contact_get_alias (contact),
 			    EMPATHY_CONTACT_LIST_STORE_COL_CONTACT, contact,
 			    EMPATHY_CONTACT_LIST_STORE_COL_IS_GROUP, FALSE,
 			    EMPATHY_CONTACT_LIST_STORE_COL_IS_SEPARATOR, FALSE,
@@ -1037,10 +1032,11 @@ contact_list_store_add_contact (EmpathyContactListStore *store,
 	GList                      *groups = NULL, *l;
 	TpConnection               *connection;
 	EmpathyContactListFlags     flags = 0;
+	char                       *protocol_name;
 
 	priv = GET_PRIV (store);
 
-	if (EMP_STR_EMPTY (empathy_contact_get_name (contact)) ||
+	if (EMP_STR_EMPTY (empathy_contact_get_alias (contact)) ||
 	    (!priv->show_offline && !empathy_contact_is_online (contact))) {
 		return;
 	}
@@ -1054,69 +1050,54 @@ contact_list_store_add_contact (EmpathyContactListStore *store,
 		flags = empathy_contact_manager_get_flags_for_connection (
 			EMPATHY_CONTACT_MANAGER (priv->list), connection);
 	}
+
+	tp_connection_parse_object_path (connection, &protocol_name, NULL);
+
 	if (!groups) {
-#if HAVE_FAVOURITE_CONTACTS
-		GtkTreeIter iter_group;
+		GtkTreeIter iter_group, *parent;
 
-		contact_list_store_get_group (store, EMPATHY_CONTACT_LIST_STORE_UNGROUPED,
-			&iter_group, NULL, NULL, TRUE);
+		parent = &iter_group;
 
-		gtk_tree_store_insert_after (GTK_TREE_STORE (store), &iter,
-					     &iter_group, NULL);
-#else
-		/* FIXME: remove this in 2.31.x */
-		/* If no groups just add it at the top level. */
-		GtkTreeModel *model = GTK_TREE_MODEL (store);
+		if (!priv->show_groups) {
+			parent = NULL;
+		} else if (!tp_strdiff (protocol_name, "local-xmpp")) {
+			/* these are People Nearby */
+			contact_list_store_get_group (store,
+				EMPATHY_CONTACT_LIST_STORE_PEOPLE_NEARBY,
+				&iter_group, NULL, NULL, TRUE);
+		} else {
+			contact_list_store_get_group (store,
+				EMPATHY_CONTACT_LIST_STORE_UNGROUPED,
+				&iter_group, NULL, NULL, TRUE);
+		}
 
-		if (gtk_tree_model_get_iter_first (model, &iter)) do {
-			EmpathyContact *c;
-
-			gtk_tree_model_get (model, &iter,
-				EMPATHY_CONTACT_LIST_STORE_COL_CONTACT, &c,
-				-1);
-
-			if (c == contact) {
-				g_object_unref (c);
-				return;
-			}
-			if (c != NULL)
-				g_object_unref (c);
-		} while (gtk_tree_model_iter_next (model, &iter));
-
-		gtk_tree_store_append (GTK_TREE_STORE (store), &iter, NULL);
-#endif
-
-		add_contact_to_store (GTK_TREE_STORE (store), &iter, contact, flags);
+		add_contact_to_store (GTK_TREE_STORE (store), &iter, parent,
+				      contact, flags);
 	}
 
+	g_free (protocol_name);
+
 	/* Else add to each group. */
 	for (l = groups; l; l = l->next) {
 		GtkTreeIter iter_group;
 
 		contact_list_store_get_group (store, l->data, &iter_group, NULL, NULL, FALSE);
 
-		gtk_tree_store_insert_after (GTK_TREE_STORE (store), &iter,
-					     &iter_group, NULL);
-
-		add_contact_to_store (GTK_TREE_STORE (store), &iter, contact, flags);
+		add_contact_to_store (GTK_TREE_STORE (store), &iter, &iter_group, contact, flags);
 		g_free (l->data);
 	}
 	g_list_free (groups);
 
-#ifdef HAVE_FAVOURITE_CONTACTS
-	if (empathy_contact_list_is_favourite (priv->list, contact)) {
+	if (priv->show_groups &&
+	    empathy_contact_list_is_favourite (priv->list, contact)) {
 	/* Add contact to the fake 'Favorites' group */
 		GtkTreeIter iter_group;
 
 		contact_list_store_get_group (store, EMPATHY_CONTACT_LIST_STORE_FAVORITE,
 			&iter_group, NULL, NULL, TRUE);
 
-		gtk_tree_store_insert_after (GTK_TREE_STORE (store), &iter,
-					     &iter_group, NULL);
-
-		add_contact_to_store (GTK_TREE_STORE (store), &iter, contact, flags);
+		add_contact_to_store (GTK_TREE_STORE (store), &iter, &iter_group, contact, flags);
 	}
-#endif
 
 	contact_list_store_contact_update (store, contact);
 }
@@ -1125,12 +1106,9 @@ static void
 contact_list_store_remove_contact (EmpathyContactListStore *store,
 				   EmpathyContact          *contact)
 {
-	EmpathyContactListStorePriv *priv;
 	GtkTreeModel               *model;
 	GList                      *iters, *l;
 
-	priv = GET_PRIV (store);
-
 	iters = contact_list_store_find_contact (store, contact);
 	if (!iters) {
 		return;
@@ -1201,7 +1179,7 @@ contact_list_store_contact_update (EmpathyContactListStore *store,
 	if (!in_list && !should_be_in_list) {
 		/* Nothing to do. */
 		DEBUG ("Contact:'%s' in list:NO, should be:NO",
-			empathy_contact_get_name (contact));
+			empathy_contact_get_alias (contact));
 
 		g_list_foreach (iters, (GFunc) gtk_tree_iter_free, NULL);
 		g_list_free (iters);
@@ -1209,7 +1187,7 @@ contact_list_store_contact_update (EmpathyContactListStore *store,
 	}
 	else if (in_list && !should_be_in_list) {
 		DEBUG ("Contact:'%s' in list:YES, should be:NO",
-			empathy_contact_get_name (contact));
+			empathy_contact_get_alias (contact));
 
 		if (priv->show_active) {
 			do_remove = TRUE;
@@ -1225,7 +1203,7 @@ contact_list_store_contact_update (EmpathyContactListStore *store,
 	}
 	else if (!in_list && should_be_in_list) {
 		DEBUG ("Contact:'%s' in list:NO, should be:YES",
-			empathy_contact_get_name (contact));
+			empathy_contact_get_alias (contact));
 
 		contact_list_store_add_contact (store, contact);
 
@@ -1236,7 +1214,7 @@ contact_list_store_contact_update (EmpathyContactListStore *store,
 		}
 	} else {
 		DEBUG ("Contact:'%s' in list:YES, should be:YES",
-			empathy_contact_get_name (contact));
+			empathy_contact_get_alias (contact));
 
 		/* Get online state before. */
 		if (iters && g_list_length (iters) > 0) {
@@ -1276,9 +1254,12 @@ contact_list_store_contact_update (EmpathyContactListStore *store,
 				    EMPATHY_CONTACT_LIST_STORE_COL_ICON_STATUS, pixbuf_status,
 				    EMPATHY_CONTACT_LIST_STORE_COL_PIXBUF_AVATAR, pixbuf_avatar,
 				    EMPATHY_CONTACT_LIST_STORE_COL_PIXBUF_AVATAR_VISIBLE, show_avatar,
-				    EMPATHY_CONTACT_LIST_STORE_COL_NAME, empathy_contact_get_name (contact),
-				    EMPATHY_CONTACT_LIST_STORE_COL_STATUS, empathy_contact_get_status (contact),
-				    EMPATHY_CONTACT_LIST_STORE_COL_STATUS_VISIBLE, !priv->is_compact,
+				    EMPATHY_CONTACT_LIST_STORE_COL_NAME, empathy_contact_get_alias (contact),
+				    EMPATHY_CONTACT_LIST_STORE_COL_PRESENCE_TYPE,
+				      empathy_contact_get_presence (contact),
+				    EMPATHY_CONTACT_LIST_STORE_COL_STATUS,
+				      empathy_contact_get_presence_message (contact),
+				    EMPATHY_CONTACT_LIST_STORE_COL_COMPACT, priv->is_compact,
 				    EMPATHY_CONTACT_LIST_STORE_COL_IS_GROUP, FALSE,
 				    EMPATHY_CONTACT_LIST_STORE_COL_IS_ONLINE, now_online,
 				    EMPATHY_CONTACT_LIST_STORE_COL_IS_SEPARATOR, FALSE,
@@ -1321,7 +1302,7 @@ contact_list_store_contact_updated_cb (EmpathyContact          *contact,
 				       EmpathyContactListStore *store)
 {
 	DEBUG ("Contact:'%s' updated, checking roster is in sync...",
-		empathy_contact_get_name (contact));
+		empathy_contact_get_alias (contact));
 
 	contact_list_store_contact_update (store, contact);
 }
@@ -1332,11 +1313,9 @@ contact_list_store_contact_set_active (EmpathyContactListStore *store,
 				       gboolean                active,
 				       gboolean                set_changed)
 {
-	EmpathyContactListStorePriv *priv;
 	GtkTreeModel               *model;
 	GList                      *iters, *l;
 
-	priv = GET_PRIV (store);
 	model = GTK_TREE_MODEL (store);
 
 	iters = contact_list_store_find_contact (store, contact);
@@ -1369,7 +1348,7 @@ contact_list_store_contact_active_new (EmpathyContactListStore *store,
 	ShowActiveData *data;
 
 	DEBUG ("Contact:'%s' now active, and %s be removed",
-		empathy_contact_get_name (contact),
+		empathy_contact_get_alias (contact),
 		remove_ ? "WILL" : "WILL NOT");
 
 	data = g_slice_new0 (ShowActiveData);
@@ -1401,12 +1380,12 @@ contact_list_store_contact_active_cb (ShowActiveData *data)
 	    !priv->show_offline &&
 	    !empathy_contact_is_online (data->contact)) {
 		DEBUG ("Contact:'%s' active timeout, removing item",
-			empathy_contact_get_name (data->contact));
+			empathy_contact_get_alias (data->contact));
 		contact_list_store_remove_contact (data->store, data->contact);
 	}
 
 	DEBUG ("Contact:'%s' no longer active",
-		empathy_contact_get_name (data->contact));
+		empathy_contact_get_alias (data->contact));
 
 	contact_list_store_contact_set_active (data->store,
 					       data->contact,
@@ -1455,14 +1434,11 @@ contact_list_store_get_group (EmpathyContactListStore *store,
 			      gboolean               *created,
 			      gboolean               is_fake_group)
 {
-	EmpathyContactListStorePriv *priv;
 	GtkTreeModel                *model;
 	GtkTreeIter                  iter_group;
 	GtkTreeIter                  iter_separator;
 	FindGroup                    fg;
 
-	priv = GET_PRIV (store);
-
 	memset (&fg, 0, sizeof (fg));
 
 	fg.name = name;
@@ -1477,8 +1453,7 @@ contact_list_store_get_group (EmpathyContactListStore *store,
 			*created = TRUE;
 		}
 
-		gtk_tree_store_append (GTK_TREE_STORE (store), &iter_group, NULL);
-		gtk_tree_store_set (GTK_TREE_STORE (store), &iter_group,
+		gtk_tree_store_insert_with_values (GTK_TREE_STORE (store), &iter_group, NULL, 0,
 				    EMPATHY_CONTACT_LIST_STORE_COL_ICON_STATUS, NULL,
 				    EMPATHY_CONTACT_LIST_STORE_COL_NAME, name,
 				    EMPATHY_CONTACT_LIST_STORE_COL_IS_GROUP, TRUE,
@@ -1491,10 +1466,7 @@ contact_list_store_get_group (EmpathyContactListStore *store,
 			*iter_group_to_set = iter_group;
 		}
 
-		gtk_tree_store_append (GTK_TREE_STORE (store),
-				       &iter_separator,
-				       &iter_group);
-		gtk_tree_store_set (GTK_TREE_STORE (store), &iter_separator,
+		gtk_tree_store_insert_with_values (GTK_TREE_STORE (store), &iter_separator, &iter_group, 0,
 				    EMPATHY_CONTACT_LIST_STORE_COL_IS_SEPARATOR, TRUE,
 				    -1);
 
@@ -1526,6 +1498,20 @@ contact_list_store_get_group (EmpathyContactListStore *store,
 	}
 }
 
+static gint
+get_position (const char **strv,
+	      const char *str)
+{
+	int i;
+
+	for (i = 0; strv[i] != NULL; i++) {
+		if (!tp_strdiff (strv[i], str))
+			return i;
+	}
+
+	return -1;
+}
+
 static gint
 compare_separator_and_groups (gboolean is_separator_a,
 			      gboolean is_separator_b,
@@ -1536,6 +1522,18 @@ compare_separator_and_groups (gboolean is_separator_a,
 			      gboolean fake_group_a,
 			      gboolean fake_group_b)
 {
+	/* these two lists are the sorted list of fake groups to include at the
+	 * top and bottom of the roster */
+	const char *top_groups[] = {
+		EMPATHY_CONTACT_LIST_STORE_FAVORITE,
+		NULL
+	};
+
+	const char *bottom_groups[] = {
+		EMPATHY_CONTACT_LIST_STORE_UNGROUPED,
+		NULL
+	};
+
 	if (is_separator_a || is_separator_b) {
 		/* We have at least one separator */
 		if (is_separator_a) {
@@ -1551,24 +1549,81 @@ compare_separator_and_groups (gboolean is_separator_a,
 	} else if (contact_a && !contact_b) {
 		return -1;
 	} else if (!contact_a && !contact_b) {
-		/* Two groups. The 'Ungrouped' fake group is display at the bottom of the
-		 * contact list and the 'Favorites' at the top. */
-		if (fake_group_a && !tp_strdiff (name_a, EMPATHY_CONTACT_LIST_STORE_UNGROUPED))
-			return 1;
-		else if (fake_group_b && !tp_strdiff (name_b, EMPATHY_CONTACT_LIST_STORE_UNGROUPED))
+		gboolean a_in_top, b_in_top, a_in_bottom, b_in_bottom;
+
+		a_in_top = fake_group_a &&
+			tp_strv_contains (top_groups, name_a);
+		b_in_top = fake_group_b &&
+			tp_strv_contains (top_groups, name_b);
+		a_in_bottom = fake_group_a &&
+			tp_strv_contains (bottom_groups, name_a);
+		b_in_bottom = fake_group_b &&
+			tp_strv_contains (bottom_groups, name_b);
+
+		if (a_in_top && b_in_top) {
+			/* compare positions */
+			return CLAMP (get_position (top_groups, name_a) -
+				      get_position (top_groups, name_b),
+				      -1, 1);
+		} else if (a_in_bottom && b_in_bottom) {
+			/* compare positions */
+			return CLAMP (get_position (bottom_groups, name_a) -
+				      get_position (bottom_groups, name_b),
+				      -1, 1);
+		} else if (a_in_top || b_in_bottom) {
 			return -1;
-		else if (fake_group_a && !tp_strdiff (name_a, EMPATHY_CONTACT_LIST_STORE_FAVORITE))
-			return -1;
-		else if (fake_group_b && !tp_strdiff (name_b, EMPATHY_CONTACT_LIST_STORE_FAVORITE))
+		} else if (b_in_top || a_in_bottom) {
 			return 1;
-		else
+		} else {
 			return g_utf8_collate (name_a, name_b);
+		}
 	}
 
 	/* Two contacts, ordering depends of the sorting policy */
 	return 0;
 }
 
+static gint
+contact_list_store_contact_sort (EmpathyContact *contact_a,
+				 EmpathyContact *contact_b)
+{
+	TpAccount *account_a, *account_b;
+	gint ret_val;
+
+	g_return_val_if_fail (contact_a != NULL || contact_b != NULL, 0);
+
+	/* alias */
+	ret_val = g_utf8_collate (empathy_contact_get_alias (contact_a),
+				  empathy_contact_get_alias (contact_b));
+
+	if (ret_val != 0)
+		goto out;
+
+	/* identifier */
+	ret_val = g_utf8_collate (empathy_contact_get_id (contact_a),
+				  empathy_contact_get_id (contact_b));
+
+	if (ret_val != 0)
+		goto out;
+
+	account_a = empathy_contact_get_account (contact_a);
+	account_b = empathy_contact_get_account (contact_b);
+
+	/* protocol */
+	ret_val = strcmp (tp_account_get_protocol (account_a),
+			  tp_account_get_protocol (account_b));
+
+	if (ret_val != 0)
+		goto out;
+
+	/* account ID */
+	ret_val = strcmp (tp_proxy_get_object_path (account_a),
+			  tp_proxy_get_object_path (account_b));
+
+out:
+	return ret_val;
+}
+
 static gint
 contact_list_store_state_sort_func (GtkTreeModel *model,
 				    GtkTreeIter  *iter_a,
@@ -1594,10 +1649,9 @@ contact_list_store_state_sort_func (GtkTreeModel *model,
 			    EMPATHY_CONTACT_LIST_STORE_COL_IS_FAKE_GROUP, &fake_group_b,
 			    -1);
 
-	ret_val = compare_separator_and_groups (is_separator_a, is_separator_b,
-		name_a, name_b, contact_a, contact_b, fake_group_a, fake_group_b);
-
-	if (ret_val != 0) {
+	if (contact_a == NULL || contact_b == NULL) {
+		ret_val = compare_separator_and_groups (is_separator_a, is_separator_b,
+			name_a, name_b, contact_a, contact_b, fake_group_a, fake_group_b);
 		goto free_and_out;
 	}
 
@@ -1609,8 +1663,8 @@ contact_list_store_state_sort_func (GtkTreeModel *model,
 		empathy_contact_get_presence (EMPATHY_CONTACT (contact_b)));
 
 	if (ret_val == 0) {
-		/* Fallback: compare by name */
-		ret_val = g_utf8_collate (name_a, name_b);
+		/* Fallback: compare by name et al. */
+		ret_val = contact_list_store_contact_sort (contact_a, contact_b);
 	}
 
 free_and_out:
@@ -1653,11 +1707,11 @@ contact_list_store_name_sort_func (GtkTreeModel *model,
 			    EMPATHY_CONTACT_LIST_STORE_COL_IS_FAKE_GROUP, &fake_group_b,
 			    -1);
 
-	ret_val = compare_separator_and_groups (is_separator_a, is_separator_b,
-		name_a, name_b, contact_a, contact_b, fake_group_a, fake_group_b);
-
-	if (ret_val == 0)
-		ret_val = g_utf8_collate (name_a, name_b);
+	if (contact_a == NULL || contact_b == NULL)
+		ret_val = compare_separator_and_groups (is_separator_a, is_separator_b,
+			name_a, name_b, contact_a, contact_b, fake_group_a, fake_group_b);
+	else
+		ret_val = contact_list_store_contact_sort (contact_a, contact_b);
 
 	if (contact_a) {
 		g_object_unref (contact_a);
@@ -1698,13 +1752,10 @@ static GList *
 contact_list_store_find_contact (EmpathyContactListStore *store,
 				 EmpathyContact          *contact)
 {
-	EmpathyContactListStorePriv *priv;
 	GtkTreeModel              *model;
 	GList                     *l = NULL;
 	FindContact                fc;
 
-	priv = GET_PRIV (store);
-
 	memset (&fc, 0, sizeof (fc));
 
 	fc.contact = contact;
@@ -1751,7 +1802,7 @@ contact_list_store_update_list_mode_foreach (GtkTreeModel           *model,
 	gtk_tree_store_set (GTK_TREE_STORE (store), iter,
 			    EMPATHY_CONTACT_LIST_STORE_COL_ICON_STATUS, pixbuf_status,
 			    EMPATHY_CONTACT_LIST_STORE_COL_PIXBUF_AVATAR_VISIBLE, show_avatar,
-			    EMPATHY_CONTACT_LIST_STORE_COL_STATUS_VISIBLE, !priv->is_compact,
+			    EMPATHY_CONTACT_LIST_STORE_COL_COMPACT, priv->is_compact,
 			    -1);
 
 	return FALSE;
@@ -1797,9 +1848,25 @@ contact_list_store_get_contact_status_icon (EmpathyContactListStore *store,
 					    EmpathyContact *contact)
 {
 	GdkPixbuf                   *pixbuf_status = NULL;
+	EmpathyContactListStorePriv *priv;
 	const gchar                 *status_icon_name = NULL;
+	gboolean                     composing = FALSE;
+
+	priv = GET_PRIV (store);
+
+	if (EMPATHY_IS_TP_CHAT (priv->list)) {
+		if (empathy_tp_chat_get_chat_state (EMPATHY_TP_CHAT (priv->list),
+		      contact) ==
+			TP_CHANNEL_CHAT_STATE_COMPOSING)
+		composing = TRUE;
+	}
+
+	if (composing) {
+		status_icon_name = EMPATHY_IMAGE_TYPING;
+	} else {
+		status_icon_name = empathy_icon_name_for_contact (contact);
+	}
 
-	status_icon_name = empathy_icon_name_for_contact (contact);
 	if (status_icon_name == NULL)
 		return NULL;
 
