@@ -259,7 +259,6 @@ chat_window_create_label (EmpathyChatWindow *window,
 			  EmpathyChat       *chat,
 			  gboolean           is_tab_label)
 {
-	EmpathyChatWindowPriv *priv;
 	GtkWidget            *hbox;
 	GtkWidget            *name_label;
 	GtkWidget            *status_image;
@@ -268,8 +267,6 @@ chat_window_create_label (EmpathyChatWindow *window,
 	PangoAttrList        *attr_list;
 	PangoAttribute       *attr;
 
-	priv = GET_PRIV (window);
-
 	/* The spacing between the button and the label. */
 	hbox = gtk_hbox_new (FALSE, 0);
 
@@ -314,6 +311,15 @@ chat_window_create_label (EmpathyChatWindow *window,
 
 	if (is_tab_label) {
 		GtkWidget            *close_button;
+		GtkWidget *sending_spinner;
+
+		sending_spinner = gtk_spinner_new ();
+
+		gtk_box_pack_start (GTK_BOX (hbox), sending_spinner,
+			FALSE, FALSE, 0);
+		g_object_set_data (G_OBJECT (chat),
+			"chat-window-tab-sending-spinner",
+			sending_spinner);
 
 		close_button = gedit_close_button_new ();
 		g_object_set_data (G_OBJECT (chat), "chat-window-tab-close-button", close_button);
@@ -453,14 +459,14 @@ get_all_unread_messages (EmpathyChatWindowPriv *priv)
 static gchar *
 get_window_title_name (EmpathyChatWindowPriv *priv)
 {
-	const gchar *active_name;
+	gchar *active_name, *ret;
 	guint nb_chats;
 	guint current_unread_msgs;
 
 	nb_chats = g_list_length (priv->chats);
 	g_assert (nb_chats > 0);
 
-	active_name = empathy_chat_get_name (priv->current_chat);
+	active_name = empathy_chat_dup_name (priv->current_chat);
 
 	current_unread_msgs = empathy_chat_get_nb_unread_messages (
 			priv->current_chat);
@@ -468,9 +474,9 @@ get_window_title_name (EmpathyChatWindowPriv *priv)
 	if (nb_chats == 1) {
 		/* only one tab */
 		if (current_unread_msgs == 0)
-			return g_strdup (active_name);
+			ret = g_strdup (active_name);
 		else
-			return g_strdup_printf (ngettext (
+			ret = g_strdup_printf (ngettext (
 				"%s (%d unread)",
 				"%s (%d unread)", current_unread_msgs),
 				active_name, current_unread_msgs);
@@ -482,7 +488,7 @@ get_window_title_name (EmpathyChatWindowPriv *priv)
 
 		if (all_unread_msgs == 0) {
 			/* no unread message */
-			return g_strdup_printf (ngettext (
+			ret = g_strdup_printf (ngettext (
 				"%s (and %u other)",
 				"%s (and %u others)", nb_others),
 				active_name, nb_others);
@@ -490,7 +496,7 @@ get_window_title_name (EmpathyChatWindowPriv *priv)
 
 		else if (all_unread_msgs == current_unread_msgs) {
 			/* unread messages are in the current tab */
-			return g_strdup_printf (ngettext (
+			ret = g_strdup_printf (ngettext (
 				"%s (%d unread)",
 				"%s (%d unread)", current_unread_msgs),
 				active_name, current_unread_msgs);
@@ -498,7 +504,7 @@ get_window_title_name (EmpathyChatWindowPriv *priv)
 
 		else if (current_unread_msgs == 0) {
 			/* unread messages are in other tabs */
-			return g_strdup_printf (ngettext (
+			ret = g_strdup_printf (ngettext (
 				"%s (%d unread from others)",
 				"%s (%d unread from others)",
 				all_unread_msgs),
@@ -507,13 +513,17 @@ get_window_title_name (EmpathyChatWindowPriv *priv)
 
 		else {
 			/* unread messages are in all the tabs */
-			return g_strdup_printf (ngettext (
+			ret = g_strdup_printf (ngettext (
 				"%s (%d unread from all)",
 				"%s (%d unread from all)",
 				all_unread_msgs),
 				active_name, all_unread_msgs);
 		}
 	}
+
+	g_free (active_name);
+
+	return ret;
 }
 
 static void
@@ -637,7 +647,7 @@ chat_window_update_chat_tab_full (EmpathyChat *chat,
 	EmpathyChatWindow     *window;
 	EmpathyChatWindowPriv *priv;
 	EmpathyContact        *remote_contact;
-	const gchar           *name;
+	gchar                 *name;
 	const gchar           *id;
 	TpAccount             *account;
 	const gchar           *subject;
@@ -648,6 +658,8 @@ chat_window_update_chat_tab_full (EmpathyChat *chat,
 	const gchar           *icon_name;
 	GtkWidget             *tab_image;
 	GtkWidget             *menu_image;
+	GtkWidget             *sending_spinner;
+	guint                  nb_sending;
 
 	window = chat_window_find_chat (chat);
 	if (!window) {
@@ -656,7 +668,7 @@ chat_window_update_chat_tab_full (EmpathyChat *chat,
 	priv = GET_PRIV (window);
 
 	/* Get information */
-	name = empathy_chat_get_name (chat);
+	name = empathy_chat_dup_name (chat);
 	account = empathy_chat_get_account (chat);
 	subject = empathy_chat_get_subject (chat);
 	remote_contact = empathy_chat_get_remote_contact (chat);
@@ -675,6 +687,9 @@ chat_window_update_chat_tab_full (EmpathyChat *chat,
 	else if (remote_contact && empathy_chat_is_composing (chat)) {
 		icon_name = EMPATHY_IMAGE_TYPING;
 	}
+	else if (empathy_chat_is_sms_channel (chat)) {
+		icon_name = EMPATHY_IMAGE_SMS;
+	}
 	else if (remote_contact) {
 		icon_name = empathy_icon_name_for_contact (remote_contact);
 	} else {
@@ -693,6 +708,16 @@ chat_window_update_chat_tab_full (EmpathyChat *chat,
 		gtk_widget_hide (menu_image);
 	}
 
+	/* Update the sending spinner */
+	nb_sending = empathy_chat_get_n_messages_sending (chat);
+	sending_spinner = g_object_get_data (G_OBJECT (chat),
+		"chat-window-tab-sending-spinner");
+
+	g_object_set (sending_spinner,
+		"active", nb_sending > 0,
+		"visible", nb_sending > 0,
+		NULL);
+
 	/* Update tab tooltip */
 	tooltip = g_string_new (NULL);
 
@@ -703,11 +728,29 @@ chat_window_update_chat_tab_full (EmpathyChat *chat,
 		id = name;
 	}
 
+	if (empathy_chat_is_sms_channel (chat)) {
+		append_markup_printf (tooltip, "%s ", _("SMS:"));
+	}
+
 	append_markup_printf (tooltip,
 			      "<b>%s</b><small> (%s)</small>",
 			      id,
 			      tp_account_get_display_name (account));
 
+	if (nb_sending > 0) {
+		char *tmp = g_strdup_printf (
+			ngettext ("Sending %d message",
+				  "Sending %d messages",
+				  nb_sending),
+			nb_sending);
+
+		g_string_append (tooltip, "\n");
+		g_string_append (tooltip, tmp);
+
+		gtk_widget_set_tooltip_text (sending_spinner, tmp);
+		g_free (tmp);
+	}
+
 	if (!EMP_STR_EMPTY (status)) {
 		append_markup_printf (tooltip, "\n<i>%s</i>", status);
 	}
@@ -738,6 +781,8 @@ chat_window_update_chat_tab_full (EmpathyChat *chat,
 	if (priv->current_chat == chat) {
 		chat_window_update (window, update_contact_menu);
 	}
+
+	g_free (name);
 }
 
 static void
@@ -865,21 +910,24 @@ chat_window_favorite_toggled_cb (GtkToggleAction   *toggle_action,
 	EmpathyChatWindowPriv *priv = GET_PRIV (window);
 	gboolean               active;
 	TpAccount             *account;
+	gchar                 *name;
 	const gchar           *room;
 	EmpathyChatroom       *chatroom;
 
 	active = gtk_toggle_action_get_active (toggle_action);
 	account = empathy_chat_get_account (priv->current_chat);
 	room = empathy_chat_get_id (priv->current_chat);
+	name = empathy_chat_dup_name (priv->current_chat);
 
 	chatroom = empathy_chatroom_manager_ensure_chatroom (
 		     priv->chatroom_manager,
 		     account,
 		     room,
-		     empathy_chat_get_name (priv->current_chat));
+		     name);
 
 	empathy_chatroom_set_favorite (chatroom, active);
 	g_object_unref (chatroom);
+	g_free (name);
 }
 
 static void
@@ -889,21 +937,24 @@ chat_window_always_urgent_toggled_cb (GtkToggleAction   *toggle_action,
 	EmpathyChatWindowPriv *priv = GET_PRIV (window);
 	gboolean               active;
 	TpAccount             *account;
+	gchar                 *name;
 	const gchar           *room;
 	EmpathyChatroom       *chatroom;
 
 	active = gtk_toggle_action_get_active (toggle_action);
 	account = empathy_chat_get_account (priv->current_chat);
 	room = empathy_chat_get_id (priv->current_chat);
+	name = empathy_chat_dup_name (priv->current_chat);
 
 	chatroom = empathy_chatroom_manager_ensure_chatroom (
 		     priv->chatroom_manager,
 		     account,
 		     room,
-		     empathy_chat_get_name (priv->current_chat));
+		     name);
 
 	empathy_chatroom_set_always_urgent (chatroom, active);
 	g_object_unref (chatroom);
+	g_free (name);
 }
 
 static void
@@ -1086,7 +1137,6 @@ chat_window_tabs_next_activate_cb (GtkAction         *action,
 				   EmpathyChatWindow *window)
 {
 	EmpathyChatWindowPriv *priv;
-	EmpathyChat           *chat;
 	gint                  index_, numPages;
 	gboolean              wrap_around;
 
@@ -1095,7 +1145,6 @@ chat_window_tabs_next_activate_cb (GtkAction         *action,
 	g_object_get (gtk_settings_get_default (), "gtk-keynav-wrap-around",
                       &wrap_around, NULL);
 
-	chat = priv->current_chat;
 	index_ = gtk_notebook_get_current_page (GTK_NOTEBOOK (priv->notebook));
 	numPages = gtk_notebook_get_n_pages (GTK_NOTEBOOK (priv->notebook));
 
@@ -1112,7 +1161,6 @@ chat_window_tabs_previous_activate_cb (GtkAction         *action,
 				   EmpathyChatWindow *window)
 {
 	EmpathyChatWindowPriv *priv;
-	EmpathyChat           *chat;
 	gint                  index_, numPages;
 	gboolean              wrap_around;
 
@@ -1121,7 +1169,6 @@ chat_window_tabs_previous_activate_cb (GtkAction         *action,
 	g_object_get (gtk_settings_get_default (), "gtk-keynav-wrap-around",
                       &wrap_around, NULL);
 
-	chat = priv->current_chat;
 	index_ = gtk_notebook_get_current_page (GTK_NOTEBOOK (priv->notebook));
 	numPages = gtk_notebook_get_n_pages (GTK_NOTEBOOK (priv->notebook));
 
@@ -1357,15 +1404,16 @@ chat_window_show_or_update_notification (EmpathyChatWindow *window,
 static void
 chat_window_set_highlight_room_labels (EmpathyChat *chat)
 {
-	gchar *markup;
+	gchar *markup, *name;
 	GtkWidget *widget;
 
 	if (!empathy_chat_is_room (chat))
 		return;
 
+	name = empathy_chat_dup_name (chat);
 	markup = g_markup_printf_escaped (
 		"<span color=\"red\" weight=\"bold\">%s</span>",
-		empathy_chat_get_name (chat));
+		name);
 
 	widget = g_object_get_data (G_OBJECT (chat), "chat-window-tab-label");
 	gtk_label_set_markup (GTK_LABEL (widget), markup);
@@ -1373,6 +1421,7 @@ chat_window_set_highlight_room_labels (EmpathyChat *chat)
 	widget = g_object_get_data (G_OBJECT (chat), "chat-window-menu-label");
 	gtk_label_set_markup (GTK_LABEL (widget), markup);
 
+	g_free (name);
 	g_free (markup);
 }
 
@@ -1512,24 +1561,15 @@ notebook_create_window_cb (GtkNotebook *source,
 
 static void
 chat_window_page_switched_cb (GtkNotebook      *notebook,
-			      gpointer          ignore, /* see note below */
+			      GtkWidget         *child,
 			      gint	        page_num,
 			      EmpathyChatWindow *window)
 {
-	EmpathyChatWindowPriv *priv;
-	EmpathyChat           *chat;
-	GtkWidget            *child;
+	EmpathyChatWindowPriv *priv = GET_PRIV (window);
+	EmpathyChat           *chat = EMPATHY_CHAT (child);
 
 	DEBUG ("Page switched");
 
-	priv = GET_PRIV (window);
-
-	/* N.B. in GTK+ 3 child is passed as the first argument to the signal,
-	 * but we can't use that while trying to retain GTK+ 2.x compatibility.
-	 */
-	child = gtk_notebook_get_nth_page (notebook, page_num);
-	chat = EMPATHY_CHAT (child);
-
 	if (priv->page_added) {
 		priv->page_added = FALSE;
 		empathy_chat_scroll_down (chat);
@@ -1775,7 +1815,7 @@ chat_window_drag_data_received (GtkWidget        *widget,
 			account =
 				tp_account_manager_ensure_account (account_manager, account_id);
 			if (account != NULL)
-				chat = empathy_chat_window_find_chat (account, contact_id);
+				chat = empathy_chat_window_find_chat (account, contact_id, FALSE);
 		}
 
 		if (account == NULL) {
@@ -1786,7 +1826,7 @@ chat_window_drag_data_received (GtkWidget        *widget,
 
 		if (!chat) {
 			empathy_chat_with_contact_id (
-				account, contact_id, empathy_get_current_action_time ());
+				account, contact_id, gtk_get_current_event_time ());
 
 			g_strfreev (strv);
 			return;
@@ -2141,13 +2181,11 @@ empathy_chat_window_get_default (gboolean room)
 	}
 
 	for (l = chat_windows; l; l = l->next) {
-		EmpathyChatWindowPriv *priv;
 		EmpathyChatWindow *chat_window;
 		GtkWidget         *dialog;
 		guint nb_rooms, nb_private;
 
 		chat_window = l->data;
-		priv = GET_PRIV (chat_window);
 
 		dialog = empathy_chat_window_get_dialog (chat_window);
 
@@ -2237,6 +2275,12 @@ empathy_chat_window_add_chat (EmpathyChatWindow *window,
 	g_signal_connect (chat, "notify::remote-contact",
 			  G_CALLBACK (chat_window_chat_notify_cb),
 			  NULL);
+	g_signal_connect (chat, "notify::sms-channel",
+			  G_CALLBACK (chat_window_chat_notify_cb),
+			  NULL);
+	g_signal_connect (chat, "notify::n-messages-sending",
+			  G_CALLBACK (chat_window_chat_notify_cb),
+			  NULL);
 	chat_window_chat_notify_cb (chat);
 
 	gtk_notebook_append_page_menu (GTK_NOTEBOOK (priv->notebook), child, label, popup_label);
@@ -2340,7 +2384,8 @@ empathy_chat_window_switch_to_chat (EmpathyChatWindow *window,
 
 EmpathyChat *
 empathy_chat_window_find_chat (TpAccount   *account,
-			       const gchar *id)
+			       const gchar *id,
+			       gboolean     sms_channel)
 {
 	GList *l;
 
@@ -2360,7 +2405,8 @@ empathy_chat_window_find_chat (TpAccount   *account,
 			chat = ll->data;
 
 			if (account == empathy_chat_get_account (chat) &&
-			    !tp_strdiff (id, empathy_chat_get_id (chat))) {
+			    !tp_strdiff (id, empathy_chat_get_id (chat)) &&
+			    sms_channel == empathy_chat_is_sms_channel (chat)) {
 				return chat;
 			}
 		}
@@ -2386,11 +2432,6 @@ empathy_chat_window_present_chat (EmpathyChat *chat,
 		window = empathy_chat_window_get_default (empathy_chat_is_room (chat));
 		if (!window) {
 			window = empathy_chat_window_new ();
-
-			/* we want to display the newly created window even if we don't present
-			* it */
-			priv = GET_PRIV (window);
-			gtk_widget_show (priv->dialog);
 		}
 
 		empathy_chat_window_add_chat (window, chat);
