@@ -335,9 +335,6 @@ static void
 event_text_channel_process_func (EventPriv *event)
 {
   EmpathyTpChat *tp_chat;
-  gint64 timestamp;
-
-  timestamp = tp_user_action_time_from_x11 (gtk_get_current_event_time ());
 
   if (event->approval->handler != 0)
     {
@@ -562,11 +559,10 @@ event_manager_chat_message_received_cb (EmpathyTpChat *tp_chat,
   EmpathyMessage *message,
   EventManagerApproval *approval)
 {
-  GtkWidget       *window = empathy_main_window_dup ();
+  GtkWidget       *window;
   EmpathyContact  *sender;
   const gchar     *header;
   const gchar     *msg;
-  TpChannel       *channel;
   EventPriv       *event;
   EmpathyEventManagerPriv *priv = GET_PRIV (approval->manager);
 
@@ -575,11 +571,14 @@ event_manager_chat_message_received_cb (EmpathyTpChat *tp_chat,
   event = event_lookup_by_approval (approval->manager, approval);
 
   sender = empathy_message_get_sender (message);
+
+  /* We only want to show incoming messages */
+  if (empathy_contact_is_user (sender))
+    return;
+
   header = empathy_contact_get_alias (sender);
   msg = empathy_message_get_body (message);
 
-  channel = empathy_tp_chat_get_channel (tp_chat);
-
   if (event != NULL)
     event_update (approval->manager, event, EMPATHY_IMAGE_NEW_MESSAGE, header,
         msg);
@@ -588,6 +587,8 @@ event_manager_chat_message_received_cb (EmpathyTpChat *tp_chat,
         EMPATHY_EVENT_TYPE_CHAT, EMPATHY_IMAGE_NEW_MESSAGE, header, msg,
         approval, event_text_channel_process_func, NULL);
 
+  window = empathy_main_window_dup ();
+
   empathy_sound_manager_play (priv->sound_mgr, window,
       EMPATHY_SOUND_CONVERSATION_NEW);
 
@@ -696,13 +697,9 @@ invite_dialog_response_cb (GtkDialog *dialog,
                            gint response,
                            EventManagerApproval *approval)
 {
-  EmpathyTpChat *tp_chat;
-
   gtk_widget_destroy (GTK_WIDGET (approval->dialog));
   approval->dialog = NULL;
 
-  tp_chat = EMPATHY_TP_CHAT (approval->handler_instance);
-
   if (response != GTK_RESPONSE_OK)
     {
       /* close channel */
@@ -927,7 +924,7 @@ approve_channels (TpSimpleApprover *approver,
 
   channel_type = tp_channel_get_channel_type_id (channel);
 
-  if (channel_type == TP_IFACE_QUARK_CHANNEL_TYPE_TEXT)
+  if (TP_IS_TEXT_CHANNEL (channel))
     {
       EmpathyTpChat *tp_chat;
 
@@ -1090,7 +1087,6 @@ event_manager_presence_changed_cb (EmpathyContact *contact,
 {
   EmpathyEventManagerPriv *priv = GET_PRIV (manager);
   TpAccount *account;
-  gchar *header = NULL;
   EmpathyPresenceManager *presence_mgr;
   GtkWidget *window = empathy_main_window_dup ();
 
@@ -1114,12 +1110,11 @@ event_manager_presence_changed_cb (EmpathyContact *contact,
           if (g_settings_get_boolean (priv->gsettings_notif,
                 EMPATHY_PREFS_NOTIFICATIONS_CONTACT_SIGNOUT))
             {
-              header = g_strdup_printf ("<b>%s</b>",
-                  empathy_contact_get_alias (contact));
-
               event_manager_add (manager, NULL, contact,
-                  EMPATHY_EVENT_TYPE_PRESENCE, EMPATHY_IMAGE_AVATAR_DEFAULT,
-                  header, _("Disconnected"), NULL, NULL, NULL);
+                  EMPATHY_EVENT_TYPE_PRESENCE_OFFLINE,
+                  EMPATHY_IMAGE_AVATAR_DEFAULT,
+                  empathy_contact_get_alias (contact), _("Disconnected"),
+                  NULL, NULL, NULL);
             }
         }
     }
@@ -1136,16 +1131,14 @@ event_manager_presence_changed_cb (EmpathyContact *contact,
           if (g_settings_get_boolean (priv->gsettings_notif,
                 EMPATHY_PREFS_NOTIFICATIONS_CONTACT_SIGNIN))
             {
-              header = g_strdup_printf ("<b>%s</b>",
-                  empathy_contact_get_alias (contact));
-
               event_manager_add (manager, NULL, contact,
-                  EMPATHY_EVENT_TYPE_PRESENCE, EMPATHY_IMAGE_AVATAR_DEFAULT,
-                  header, _("Connected"), NULL, NULL, NULL);
+                  EMPATHY_EVENT_TYPE_PRESENCE_ONLINE,
+                  EMPATHY_IMAGE_AVATAR_DEFAULT,
+                  empathy_contact_get_alias (contact), _("Connected"),
+                  NULL, NULL, NULL);
             }
         }
     }
-  g_free (header);
 
 out:
   g_object_unref (presence_mgr);
